/*
 * File:   SerialDataTransfer.cpp
 * Author: Bob Alexander
 *
 * Created on November 22, 2014, 3:45 PM
 */

#include "Common.h"
#include "app.h"
#include "DisplayManager.h"
#include "CircularQueue.h"

static DRV_HANDLE bluetooth, sysUnderTest;
static struct CircularQueue fromBluetooth, fromSUT;
static char btBuf[512], sutBuf[64];

void SerialDataTransfer_Initialize()
{
    bluetooth = DRV_USART_Open(DRV_USART_INDEX_0, (DRV_IO_INTENT_READWRITE | DRV_IO_INTENT_NONBLOCKING));
    if (bluetooth == DRV_HANDLE_INVALID)
    {
        while (true) ;
    }
    CircularQueue_Create(&fromBluetooth, btBuf, countof(btBuf));
    sysUnderTest = DRV_USART_Open(DRV_USART_INDEX_1, (DRV_IO_INTENT_READWRITE | DRV_IO_INTENT_NONBLOCKING));
    if (sysUnderTest == DRV_HANDLE_INVALID)
    {
        while (true) ;
    }
    CircularQueue_Create(&fromSUT, sutBuf, countof(sutBuf));
}

void SerialDataTransfer_Tasks()
{
    DRV_USART_ERROR err;

    // Write to the SUT from the fromBluetooth buffer
    while (!CircularQueue_IsEmpty(&fromBluetooth) && !(DRV_USART_TransferStatus(sysUnderTest) & DRV_USART_TRANSFER_STATUS_TRANSMIT_FULL))
    {
        char ch;
        CircularQueue_Read(&fromBluetooth, &ch);
        DRV_USART_WriteByte(sysUnderTest, ch);
        err = DRV_USART_ErrorGet(sysUnderTest);
    }
    
    // Read from Bluetooth into the buffer
    while (DRV_USART_TransferStatus(bluetooth) & DRV_USART_TRANSFER_STATUS_RECEIVER_DATA_PRESENT)
    {
        char ch = DRV_USART_ReadByte(bluetooth);
        err = DRV_USART_ErrorGet(sysUnderTest);
        if (err == DRV_USART_ERROR_NONE)
        {
            // If the buffer is full, so be it
            CircularQueue_Write(&fromBluetooth, ch);
        }
        DisplayManager_BTCharReceived(ch);
    }

    // Write to Bluetooth from the fromSUT buffer
    while (!CircularQueue_IsEmpty(&fromSUT) && !(DRV_USART_TransferStatus(bluetooth) & DRV_USART_TRANSFER_STATUS_TRANSMIT_FULL))
    {
        char ch;
        CircularQueue_Read(&fromSUT, &ch);
        DRV_USART_WriteByte(bluetooth, ch);
        err = DRV_USART_ErrorGet(sysUnderTest);
    }

    // Read from SUT into fromSUT buffer
    while (DRV_USART_TransferStatus(sysUnderTest) & DRV_USART_TRANSFER_STATUS_RECEIVER_DATA_PRESENT)
    {
        char ch = DRV_USART_ReadByte(sysUnderTest);
        err = DRV_USART_ErrorGet(sysUnderTest);
        if (err == DRV_USART_ERROR_NONE)
        {
            // If the buffer is full, so be it
            CircularQueue_Write(&fromSUT, ch);
        }
        DisplayManager_SUTCharReceived(ch);
    }
}

void SerialDataTransfer_SetBaud(size_t baud)
{
    DRV_USART_BaudSet(sysUnderTest, baud);
}
